﻿using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Interface.DBInterface;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.Result;
using System.Text;
using WingServerCommon.Config;
using WingInterfaceLibrary.DTO.Share;
using WingServerCommon.Config.Parameters;
using ReportService.Posters;
using System.Collections.Concurrent;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.Consultation;

namespace ReportService.Service
{
    /// <summary>
    /// 报告服务
    /// </summary>
    public partial class ReportService : JsonRpcService, IReportService
    {
        private IUserDBService _userService;
        private IDeviceInfoDBService _deviceInfoDBService;
        private IAuthenticationService _authenticationService;
        private IOrganizationDBService _organizationDBService;
        private INotificationService _notificationService;
        private IRecordsInfoDBService _recordsInfoDBService;
        private IPatientInfoDBService _patientInfoDBService;
        private IReportDBService _reportDBService;
        private IAdminDBService _adminDBService;
        private IRemedicalDBService _remedicalDBService;
        private IEmailService _emailService;
        private ISMSService _smsService;
        private IShareDBService _shareDBService;
        private IRemedicalService _remedicalService;
        private IReportPosterDBService _reportPosterDBService;
        private IStorageService _storageService;
        private IConsultationRecordDBService _consultationRecordDBService;
        private string _messageUrl = ConfigurationManager.GetParammeter<StringParameter>("Report", "MessageUrl").Value;
        private string _reportUrl = ConfigurationManager.GetParammeter<StringParameter>("Report", "ReportUrl").Value;
        private string _reportMessageTemplateId = ConfigurationManager.GetParammeter<StringParameter>("Report", "ReportMessageTemplateId").Value;
        private readonly ConcurrentDictionary<string, string> _aiReportTemplateDictionary = new ConcurrentDictionary<string, string>();
        private readonly ReportPostersObserver _reportPosterObserver;
        private readonly HttpClient _httpClient = new HttpClient();
        public ReportService()
        {
            CacheMaintenance.Instance.Get<IReportPostersManager>().RegisterDBFunction(FindReportPosterByCode, FindReportPostersWithCodes);
            _reportPosterObserver = new ReportPostersObserver(FindReporByCode, UpdateReportPushRecord);
            _httpClient.Timeout = TimeSpan.FromMinutes(10);
        }

        /// <summary>
        /// Init service
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _userService = GetProxy<IUserDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _deviceInfoDBService = GetProxy<IDeviceInfoDBService>();
            _organizationDBService = GetProxy<IOrganizationDBService>();
            _notificationService = GetProxy<INotificationService>();
            _recordsInfoDBService = GetProxy<IRecordsInfoDBService>();
            _patientInfoDBService = GetProxy<IPatientInfoDBService>();
            _reportDBService = GetProxy<IReportDBService>();
            _adminDBService = GetProxy<IAdminDBService>();
            _remedicalDBService = GetProxy<IRemedicalDBService>();
            _emailService = GetProxy<IEmailService>();
            _smsService = GetProxy<ISMSService>();
            _shareDBService = GetProxy<IShareDBService>();
            _remedicalService = GetProxy<IRemedicalService>();
            _reportPosterDBService = GetProxy<IReportPosterDBService>();
            _storageService = GetProxy<IStorageService>();
            LoadReportPosterFromDb();
        }

        /// <summary>
        /// 通用异常处理方法
        /// </summary>
        /// <param name="customerRpcCodeEnum">异常标识码</param>
        /// <param name="msg">异常信息</param>
        /// <param name="internalMessage">内部异常信息，非必填</param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg, string internalMessage = null)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg, internalMessage);
        }

        /// <summary>
        /// 查询报告模板详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> FindReportTemplateByCodeAsync(FindReportTemplateByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var detailInfo = await _reportDBService.GetReportTemplateAsync(new GetReportTemplateDBRequest { ReportTemplateCode = request.ReportTemplateCode });
            if (detailInfo != null)
            {
                var language = "zh-CN";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }

                if (detailInfo.TemplatePreviewList?.Count > 0)
                {
                    detailInfo.TemplatePreviewList.ForEach(x =>
                    {
                        if (x.Language == language)
                        {
                            if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                            {
                                detailInfo.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                            }
                        }
                    });
                    detailInfo.TemplatePreviewList = null;
                }
            }
            return detailInfo;
        }

        /// <summary>
        /// 查询报告详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportDTO> FindReportByCodeAsync(FindReportByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var reportInfo = await FindReportByCodeAsync(request.ReportCode, request.Language);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            reportInfo.PatientName = CryptionHelper.NameEncrypt(reportInfo.PatientName, rootOrg.IsEncryptedShow);
            return reportInfo;
        }

        private async Task<ReportDTO> FindReportByCodeAsync(string reportCode, string language)
        {
            var reportInfo = await _reportDBService.GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = reportCode });
            if (reportInfo.DiagnosisLabels.Contains("VAid"))
            {
                if (string.IsNullOrWhiteSpace(language))
                {
                    language = "zh-CN";
                }
                var newReportDatasJson = await _remedicalService.GetReportElementByLanguageAsync(new GetReportElementByLanguageRequest
                {
                    ReportDatasJosn = reportInfo.ReportDatasJson,
                    Language = language,
                });
                reportInfo.ReportDatasJson = newReportDatasJson;
                reportInfo.ReportTemplateJson = GetAIReportTemplate(language);
            }
            return reportInfo;
        }

        /// <summary>
        /// 查询报告分享地址
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> FindReportShareUrlAsync(FindReportShareUrlRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var reportCode = request.ReportCode;
            return string.Format(_reportUrl, reportCode, !string.IsNullOrWhiteSpace(request.LanguageCode) ? request.LanguageCode : "zh-CN");
        }

        private async Task CheckReportConverter(string reportCode)
        {
            var reportInfo = await FindReportByCodeAsync(reportCode, "");
            var reportPreviewList = reportInfo.ReportPreviewList?.Where(x => x.FileType == UploadFileTypeEnum.JPG);
            if (reportPreviewList == null || !reportPreviewList.Any())
            {
                try
                {
                    DoReportConverter(reportCode, UploadFileTypeEnum.JPG);
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"check report convert, create report jpg failed, report code:{reportCode}");
                }
            }
        }

        /// <summary>
        /// 查询检查关联报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportDTO>> FindExamReportsAsync(FindExamReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            var result = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
            foreach (var item in result)
            {
                item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, rootOrg.IsEncryptedShow);
            }
            return result;
        }

        public async Task<List<ReportDTO>> FindPatientReportsAsync(FindPatientReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            var result = await _reportDBService.GetReportByPatientCodeAsync(request.PatientCode);
            foreach (var item in result)
            {
                item.CanEditReport = item.DiagnosisLabels.Contains("VAid") || item.ReportUserCode == userCode;
                item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, rootOrg.IsEncryptedShow);
            }
            return result;
        }

        /// <summary>
        /// 查询报告标签配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportLabelDTO>> FindReportLabelsAsync(FindReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var usageType = ReportLabelUsageTypeEnum.People;
            var organzation = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (organzation != null)
            {
                usageType = (ReportLabelUsageTypeEnum)organzation.PatientType;
            }
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
            }
            return await _reportDBService.GetReportLabelsAsync(new GetReportLabelsDBRequest { LanguageCode = request.LanguageCode, UsageType = usageType });
        }

        /// <summary>
        /// 根据分享短码查询分享内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ShareDTO> FindShareContentAsync(FindShareContentRequest request)
        {
            return await _shareDBService.GetShareInfoAsync(new GetShareInfoDBRequest { ShortCode = request.ShortCode });
        }

        /// <summary>
        /// 创建报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddReportAsync(AddReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var deviceCode = "";
            var patientCode = "";
            var record = new RecordInfoDTO();
            var consultationRecord = new ConsultationDetailDTO();
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
                case ReportTypeEnum.LiveConsultation:
                    var dbRequest = new ConsultationRecordCodeDBRequest { Code = request.RecordCode };
                    consultationRecord = await _consultationRecordDBService.FindConsultationRecordByCodeAsync(dbRequest);
                    if (consultationRecord == null || string.IsNullOrWhiteSpace(consultationRecord.ConsultationCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = consultationRecord.DeviceCode;
                    patientCode = consultationRecord.PatientCode;
                    break;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !request.DiagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            if (!string.IsNullOrWhiteSpace(request.ReferralRecordCode))
            {
                var referralRecordInfo = await _remedicalDBService.GetReferralRecordInfoAsync(new GetReferralRecordByCodeDBRequest
                {
                    Code = request.ReferralRecordCode
                });
                if (referralRecordInfo == null || string.IsNullOrWhiteSpace(referralRecordInfo.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNotExisted, "Referral Record Not Existed");
                }
                if (referralRecordInfo.ReferralStatus != ReferralStatusEnum.Accepted)
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNeverAccepted, "Referral Record Never Accepted");
                }
            }

            var patientId = GetPatientId(request.ReportDatasJson);
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson,
                ReportDatasJson = request.ReportDatasJson,
                PatientId = patientId,
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = request.DiagnosisLabels ?? new List<string>(),
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
            });
            if (string.IsNullOrEmpty(reportId))
            {
                ThrowCustomerException(CustomerRpcCode.AddReportFailed, "Add report failed");
            }
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    if (record.RecordStatus != RecordStatusEnum.Completed)
                    {
                        record.RecordStatus = RecordStatusEnum.Completed;
                        await _recordsInfoDBService.UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
                    }
                    break;
                case ReportTypeEnum.LiveConsultation:
                    if (consultationRecord.ConsultationStatus != ConsultationStatusEnum.End)
                    {
                        consultationRecord.ConsultationStatus = ConsultationStatusEnum.End;
                        await _consultationRecordDBService.UpdateConsultationRecordAsync(consultationRecord);
                    }
                    break;
            }
            var reportPosters = _reportPosterObserver.GetDevicePosters(deviceCode);
            if (reportPosters != null && reportPosters.Count > 0)
            {
                foreach (var item in reportPosters)
                {
                    var addReportPushRecordDBRequest = new AddReportPushRecordDBRequest()
                    {
                        Data = new ReportPushRecordDTO()
                        {
                            PatientId = patientId,
                            PatientName = request.PatientName,
                            PoseterConfigId = item.Code,
                            State = PushDataStateEnum.UnDo,
                            PoseterConfigName = item.ReceiverName,
                            ReportId = reportId,
                            PatientRecordId = request.RecordCode,
                            PushResultToken = string.Empty,
                            PushResponse = string.Empty,
                        }

                    };
                    var reportPushRecord = await _reportPosterDBService.AddReportPushRecordAsync(addReportPushRecordDBRequest);
                    if (reportPushRecord != null)
                    {
                        JoinQueue(reportPushRecord);
                    }
                }
            }
            JoinQueue(reportId);
            return reportId;
        }

        /// <summary>
        /// 修改报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportAsync(ModifyReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var deviceCode = "";
            var patientCode = "";
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
                case ReportTypeEnum.LiveConsultation:
                    var dbRequest = new ConsultationRecordCodeDBRequest { Code = request.RecordCode };
                    var consultationRecord = await _consultationRecordDBService.FindConsultationRecordByCodeAsync(dbRequest);
                    if (consultationRecord == null || string.IsNullOrWhiteSpace(consultationRecord.ConsultationCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = consultationRecord.DeviceCode;
                    patientCode = consultationRecord.PatientCode;
                    break;
            }
            var diagnosisLabels = request.DiagnosisLabels ?? new List<string>();
            if (request.ReportOrgan != DiagnosisOrganEnum.Null && !string.IsNullOrWhiteSpace(userCode))
            {
                diagnosisLabels.Remove("VAid");
                var examReportList = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
                var userOrganReport = examReportList.FirstOrDefault(x => x.ReportUserCode == userCode && x.ReportOrgan == request.ReportOrgan);
                request.ReportCode = userOrganReport?.ReportCode ?? string.Empty;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !diagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            if (!string.IsNullOrWhiteSpace(request.ReferralRecordCode))
            {
                var referralRecordInfo = await _remedicalDBService.GetReferralRecordInfoAsync(new GetReferralRecordByCodeDBRequest
                {
                    Code = request.ReferralRecordCode
                });
                if (referralRecordInfo == null || string.IsNullOrWhiteSpace(referralRecordInfo.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNotExisted, "Referral Record Not Existed");
                }
                if (referralRecordInfo.ReferralStatus != ReferralStatusEnum.Accepted)
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNeverAccepted, "Referral Record Never Accepted");
                }
            }
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                ReportCode = request.ReportCode,
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson,
                ReportDatasJson = request.ReportDatasJson,
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = diagnosisLabels,
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
            });

            if (!string.IsNullOrEmpty(reportId))
            {
                var reportPosters = _reportPosterObserver.GetDevicePosters(deviceCode);
                if (reportPosters != null && reportPosters.Count > 0)
                {
                    foreach (var item in reportPosters)
                    {
                        var reportPushRecord = await _reportPosterDBService.
                        UpdatePushRecordStateByReportIdAsync(reportId, item.Code, PushDataStateEnum.UnDo);
                        if (reportPushRecord != null)
                        {
                            JoinQueue(reportPushRecord);
                        }
                    }
                }
                JoinQueue(reportId);
                return true;
            }

            return false;
        }

        /// <summary>
        /// 修改报告标签
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportLabelsAsync(ModifyReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }

            var result = await _reportDBService.SaveReportLabelsAsync(new SaveReportLabelsDBRequest
            {
                ReportCode = request.ReportCode,
                ReportLabels = request.ReportLabels ?? new List<string>()
            });
            return result;
        }

        /// <summary>
        /// 分享报告-短信
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportBySmsAsync(ShareReportBySmsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            if (request.MobileList?.Any() != true)
            {
                ThrowCustomerException(CustomerRpcCode.MobileListEmpty, "MobileList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.MessageUrlEmpty, "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode, request.LanguageCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                ThrowCustomerException(CustomerRpcCode.ReportPageNeverConfig, "Report Url empty error");
            }
            if (string.IsNullOrWhiteSpace(_reportMessageTemplateId))
            {
                ThrowCustomerException(CustomerRpcCode.ReportSMSTempalteNeverConfig, "SMS Template Id empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                ThrowCustomerException(CustomerRpcCode.CreateShareCodeError, "ShareCode empty error");
            }

            return await _smsService.GeneralMessageAsync(request.MobileList, _reportMessageTemplateId, new List<string> { shortCode });
        }

        /// <summary>
        /// 分享报告-邮件
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportByEmailAsync(ShareReportByEmailRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            if (request.EmailList?.Any() != true)
            {
                ThrowCustomerException(CustomerRpcCode.EmailListEmpty, "EmailList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.MessageUrlEmpty, "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode, request.LanguageCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                ThrowCustomerException(CustomerRpcCode.ReportPageNeverConfig, "Report Url empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportSMSTempalteNeverConfig, "SMS Template Id empty error");
            }

            return await _emailService.SendEmailAsync(request.LanguageCode, request.EmailList, "ShareReport", _messageUrl, shortCode);
        }

        /// <summary>
        /// 获取病人id
        /// </summary>
        /// <param name="reportDatasJson"></param>
        /// <returns></returns>
        private string GetPatientId(string reportDatasJson)
        {
            try
            {
                if (!string.IsNullOrWhiteSpace(reportDatasJson))
                {
                    var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportDatasJson, new ReportElementDTOConverter());
                    foreach (var r in reportElements)
                    {
                        if (r is ReportElementStringDTO es && es.Name.ToLower() == "patientid")
                        {
                            return es.Value?.ToString() ?? string.Empty;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"get patient id err, {ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 获取邮箱Base64字符串
        /// </summary>
        /// <param name="content"></param>
        /// <returns></returns>
        private string GetBase64EmailString(string content)
        {
            content = Convert.ToBase64String(Encoding.UTF8.GetBytes(content));
            var str = $"=?UTF8?B?{content}?=";
            return str;
        }

        /// <summary>
        /// 客户端用户验证
        /// </summary>
        /// <param name="request"></param>
        /// <returns>User Code</returns>
        private async Task<string> ValidateTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }

        /// <summary>
        /// 拷贝成个人报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<string> CopyPersonalReportTemplateAsync(CopyReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, userCode, string.Empty);
        }

        /// <summary>
        /// 拷贝成机构报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<string> CopyOrgReportTemplateAsync(CopyOrgReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, string.Empty, userInfo.RootOrganizationCode);
        }

        /// <summary>
        /// 获取报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            var defaultReportTemplateCode = await _userService.GetUserDefaultReportTemplateCodeAsync(userCode);
            dBRequest.PersonalCode = userCode;
            dBRequest.PageSize = request.PageSize;
            dBRequest.CurrentPage = request.PageIndex;
            dBRequest.ReportTemplateName = request.ReportTemplateName;
            dBRequest.IsDefault = request.IsDefault;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                dBRequest.OrganizationCode = userInfo.OrganizationCode;
                var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
                dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            }
            else
            {
                dBRequest.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            }
            var result = await _reportDBService.GetReportTemplatePageAsync(dBRequest);
            bool isHadUserDefault = false;
            if (!string.IsNullOrWhiteSpace(defaultReportTemplateCode))
            {
                result.PageData.ForEach(item =>
                {
                    if (item.ReportTemplateCode == defaultReportTemplateCode)
                    {
                        item.IsUserDefault = true;
                        isHadUserDefault = true;
                    }
                });
            }
            if (!isHadUserDefault)
            {
                var defaultItem = result.PageData.FirstOrDefault(f => f.IsDefault && f.ReportTemplateStateType == ReportTemplateStatusTypeEnum.Published);
                if (defaultItem != null)
                {
                    defaultItem.IsUserDefault = true;
                    await _userService.SetUserDefaultReportTemplateCodeAsync(userCode, defaultItem.ReportTemplateCode);
                }
            }
            if (result?.PageData?.Count > 0)
            {
                var language = "zh-CN";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }
                result.PageData.ForEach(item =>
                {
                    if (item.TemplatePreviewList?.Count > 0)
                    {
                        item.TemplatePreviewList.ForEach(x =>
                        {
                            if (x.Language == language)
                            {
                                if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                                {
                                    item.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                                }
                            }
                        });
                        item.TemplatePreviewList = null;
                    }
                });
            }
            return result;
        }

        /// <summary>
        /// 获取机构报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetOrgReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector || string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            dBRequest.PageSize = request.PageSize;
            dBRequest.CurrentPage = request.PageIndex;
            dBRequest.ReportTemplateName = request.ReportTemplateName;
            dBRequest.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dBRequest.IsDefault = request.IsDefault;
            var result = await _reportDBService.GetOrgReportTemplatePageAsync(dBRequest);
            if (result?.PageData?.Count > 0)
            {
                var language = "zh-CN";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }
                result.PageData.ForEach(item =>
                {
                    if (item.TemplatePreviewList?.Count > 0)
                    {
                        item.TemplatePreviewList.ForEach(x =>
                        {
                            if (x.Language == language)
                            {
                                if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                                {
                                    item.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                                }
                            }
                        });
                        item.TemplatePreviewList = null;
                    }
                });
            }
            return result;
        }

        /// <summary>
        /// 添加报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            try
            {
                var json = JsonHelper.DeserializerJson<object>(request.ReportTemplateJson);
            }
            catch (Exception ex)
            {
                ThrowCustomerException(CustomerRpcCode.JsonError, "Json format error");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.PersonalCode = userCode;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Personal;
            return await _reportDBService.AddReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 验证报告模板名称是否重复
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CheckReportTemplateName(UpdateReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            //验证报告模板名称是否重复
            var res = await _reportDBService.CheckReportTemplateName(dbRequest);
            return res;
        }

        /// <summary>
        /// 更新报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateReportTemplateAsync(UpdateReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            dbRequest.Data.ReportDatasJson = request.ReportDatasJson;
            var res = await _reportDBService.UpdateReportTemplateAsync(dbRequest);
            //添加生成队列，生成报告模板图片
            if (res)
            {
                var req = new GetReportTemplateDBRequest()
                {
                    ReportTemplateCode = request.ReportTemplateCode
                };
                await CreateReportTemplatePreviewAsync(req);
            }
            return res;
        }

        /// <summary>
        /// 删除报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateCodeEmpty, "ReportTemplate code empty error");
            }
            var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            if (ReportTemplateInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrgReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Organization;
            return await _reportDBService.AddReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 删除机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateCodeEmpty, "ReportTemplate code empty error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotDirector, "User not director");
            }
            if (ReportTemplateInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (ReportTemplateInfo.OrganizationCode != userInfo.OrganizationCode && ReportTemplateInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusAsync(AddThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.PersonalCode = userCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Personal;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddOrgThesaurusAsync(AddThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null || !userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Organization;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 更新词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusAsync(UpdateThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            dbRequest.Data.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.UpdateThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusClassifyAsync(AddThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                ThrowCustomerException(CustomerRpcCode.EditThesaurusItemError, "Thesaurus item name empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ThesaurusItemName = request.ThesaurusItemName,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusContentAsync(AddThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusClassifyAsync(UpdateThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                ThrowCustomerException(CustomerRpcCode.EditThesaurusItemError, "Thesaurus item name empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemName = request.ThesaurusItemName,
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusContentAsync(UpdateThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveThesaurusDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusItemAsync(RemoveThesaurusItemRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItemCode = request.ThesaurusItemCode;
            return await _reportDBService.RemoveThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 获取词条库（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            request.PersonalCode = userCode;
            var defaultThesaurusCode = await _userService.GetUserDefaultThesaurusCodeAsync(userCode);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                request.OrganizationCode = userInfo.OrganizationCode;
                var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
                request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            }
            else
            {
                request.ThesaurusUsageType = ThesaurusUsageTypeEnum.People;
            }
            var result = await _reportDBService.GetThesaurusPageAsync(request);
            bool isHadUserDefault = false;
            if (!string.IsNullOrWhiteSpace(defaultThesaurusCode))
            {
                result.PageData.ForEach(item =>
                {
                    if (item.ThesaurusCode == defaultThesaurusCode)
                    {
                        item.IsUserDefault = true;
                        isHadUserDefault = true;
                    }
                });
            }
            if (!isHadUserDefault)
            {
                var defaultItem = result.PageData.FirstOrDefault(f => f.IsDefault && f.ThesaurusStateType == ThesaurusStatusTypeEnum.Published);
                if (defaultItem != null)
                {
                    defaultItem.IsUserDefault = true;
                    await _userService.SetUserDefaultThesaurusCodeAsync(userCode, defaultItem.ThesaurusCode);
                }
            }
            return result;
        }

        /// <summary>
        /// 获取词条项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusItemsAsync(GetThesaurusItemsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsAsync(request);
            return list.OrderBy(s=>s.Sort).ToList();
        }

        /// <summary>
        /// 获取词条项详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusItemDTO> GetThesaurusItemByCodeAsync(GetThesaurusItemByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemByCodeAsync(request);
        }


        /// <summary>
        /// 根据父节点编码获取词条分类
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusClassifyByParentCodeAsync(GetThesaurusClassifyByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Classification);
            return list.OrderBy(s=>s.Sort).ToList();
        }

        /// <summary>
        /// 根据父节点编码获取词条内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusContentByParentCodeAsync(GetThesaurusContentByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Contents);
            return list.OrderBy(s=>s.Sort).ToList();
        }

        /// <summary>
        /// 删除机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotDirector, "User not director");
            }
            if (thesaurusInfo.IsDefault)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (thesaurusInfo.OrganizationCode != userInfo.OrganizationCode && thesaurusInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveThesaurusDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 获取机构词条库分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetOrgThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector || string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            request.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetOrgThesaurusPageAsync(request);
        }

        /// <summary>
        /// 拷贝成个人词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyPersonalThesaurusAsync(CopyThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, userCode, string.Empty);
        }

        /// <summary>
        /// 拷贝成机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyOrgThesaurusAsync(CopyOrgThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, string.Empty, userInfo.RootOrganizationCode);
        }

        public string GetAIReportTemplate(string language)
        {
            if (_aiReportTemplateDictionary.TryGetValue(language, out var reportTemplate))
            {
                return reportTemplate ?? string.Empty;
            }
            string filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, @"Resource\Diagnosis\ReportTemplate", $"{language}.json");
            if (File.Exists(filePath))
            {
                var content = File.ReadAllText(filePath) ?? string.Empty;
                _aiReportTemplateDictionary.AddOrUpdate(language, content, (d, h) => content);
                return content;
            }
            return string.Empty;
        }



        /// <summary>
        /// 获取用户默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusAllDTO> GetDefaultThesaurusContentAsync(GetDefaultThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var defaultThesaurus = await _userService.GetUserDefaultThesaurusContentAsync(userCode);
            if (defaultThesaurus == null)
            {
                var pageRequest = new GetThesaurusPageRequest();
                pageRequest.Token = request.Token;
                await GetThesaurusPageAsync(pageRequest);
                defaultThesaurus = await _userService.GetUserDefaultThesaurusContentAsync(userCode);
            }
            return defaultThesaurus;
        }

        /// <summary>
        /// 设置用户默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultThesaurusAsync(SetUserDefaultThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "ThesaurusCode is empty");
            }
            return await _userService.SetUserDefaultThesaurusCodeAsync(userCode, request.ThesaurusCode);
        }

        /// <summary>
        /// 获取用户默认报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetDefaultReportTemplateContentAsync(GetDefaultReportTemplateContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var defaultReportTemplate = await _userService.GetUserDefaultReportTemplateContentAsync(userCode);
            if (defaultReportTemplate == null)
            {
                var pageRequest = new FindReportTemplatePagesRequest();
                pageRequest.Token = request.Token;
                await GetReportTemplatePageAsync(pageRequest);
                defaultReportTemplate = await _userService.GetUserDefaultReportTemplateContentAsync(userCode);
            }
            return defaultReportTemplate;
        }

        /// <summary>
        /// 设置用户默认报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultReportTemplateAsync(SetUserDefaultReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateCode is empty");
            }
            return await _userService.SetUserDefaultReportTemplateCodeAsync(userCode, request.ReportTemplateCode);
        }

        #region 报告模板 预览 服务端生成 图片功能 
        /// <summary>
        /// 创建报告模板预览
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateReportTemplatePreviewAsync(GetReportTemplateDBRequest request)
        {
            var result = false;
            JoinTemplateQueue(request.ReportTemplateCode);
            return result;
        }
        #endregion
    }
}

